const Express = require('express')
const WorkboxPlugin = require('workbox-webpack-plugin')
var compression = require('compression')

const appConfig = require('../utils/get-app-config')
const { IS_DEV, IS_PROD } = require('../config/env.config')

const webpackDevMiddleware = require('./middlewares/webpack.dev')
const proxyMiddleware = require('./middlewares/proxy')
const ssrMiddleware = require('./middlewares/ssr')

module.exports = function createApp(options, onStart = () => {}) {
  const app = Express()

  // 路由hash配置的时候，带路由的页面刷新会404，需要加入这个
  // 不过服务端渲染的时候好像有问题，到时候再看吧
  app.use(require('connect-history-api-fallback')())
  // 开启gzip
  app.use(compression())

  // 服务端静态资源路径
  const publicPath = `${appConfig.deploy ? (appConfig.deploy.project || 'public') : 'public'}`
  app.use(`/${publicPath}`, Express.static(publicPath))

  if (IS_DEV) {
    webpackDevMiddleware(app)
  }

  if (IS_PROD && appConfig.isSSR) {
    app.use(ssrMiddleware)
  }

  // proxy配置
  app.use(proxyMiddleware)

  app.listen(appConfig.appPort, function () {
    console.log(`server is running in http://localhost:${appConfig.appPort}`)
  })

  // 服务器启动回调
  onStart()

  return app
}
